[PATCH] Fix ELF st_other flag.

Logan Chien tzuhsiang.chien at gmail.com
Wed Dec 4 08:34:23 PST 2013


Hi rafael,

ELF_Other_Weakref and ELF_Other_ThumbFunc seems to be LLVM
internal ELF symbol flags.  These should not be emitted to
object file.

This commit defines ELF_STO_Shift for the target-defined
flags for st_other, and increase the value of
ELF_Other_Shift to 16.

http://llvm-reviews.chandlerc.com/D2329

Files:
  include/llvm/MC/MCELFSymbolFlags.h
  lib/MC/ELFObjectWriter.cpp
  lib/MC/MCELF.cpp
  test/MC/ELF/thumb-st_other.s

Index: lib/MC/ELFObjectWriter.cpp
===================================================================
--- lib/MC/ELFObjectWriter.cpp
+++ lib/MC/ELFObjectWriter.cpp
@@ -551,8 +551,7 @@
   // Other and Visibility share the same byte with Visibility using the lower
   // 2 bits
   uint8_t Visibility = MCELF::GetVisibility(OrigData);
-  uint8_t Other = MCELF::getOther(OrigData) <<
-    (ELF_Other_Shift - ELF_STV_Shift);
+  uint8_t Other = MCELF::getOther(OrigData) << (ELF_STO_Shift - ELF_STV_Shift);
   Other |= Visibility;
 
   uint64_t Value = SymbolValue(Data, Layout);
Index: lib/MC/MCELF.cpp
===================================================================
--- lib/MC/MCELF.cpp
+++ lib/MC/MCELF.cpp
@@ -72,13 +72,13 @@
 // Other is stored in the last six bits of st_other
 // st_other values are stored in the second byte of get/setFlags
 void MCELF::setOther(MCSymbolData &SD, unsigned Other) {
-  uint32_t OtherFlags = SD.getFlags() & ~(0x3f << ELF_Other_Shift);
-  SD.setFlags(OtherFlags | (Other << ELF_Other_Shift));
+  uint32_t OtherFlags = SD.getFlags() & ~(0x3f << ELF_STO_Shift);
+  SD.setFlags(OtherFlags | (Other << ELF_STO_Shift));
 }
 
 unsigned MCELF::getOther(MCSymbolData &SD) {
   unsigned Other =
-    (SD.getFlags() & (0x3f << ELF_Other_Shift)) >> ELF_Other_Shift;
+    (SD.getFlags() & (0x3f << ELF_STO_Shift)) >> ELF_STO_Shift;
   return Other;
 }
 
Index: test/MC/ELF/thumb-st_other.s
===================================================================
--- /dev/null
+++ test/MC/ELF/thumb-st_other.s
@@ -0,0 +1,19 @@
+@ Check the value of st_other for thumb function.
+
+@ ARM does not define any st_other flags for thumb function.  The value
+@ for st_other should always be 0.
+
+@ RUN: llvm-mc < %s -triple thumbv5-linux-gnueabi -filetype=obj -o - \
+@ RUN:   | llvm-readobj -t | FileCheck %s
+
+	.syntax	unified
+	.text
+	.align	2
+	.thumb_func
+	.global	main
+	.type	main,%function
+main:
+	bx	lr
+
+@ CHECK: Name: main
+@ CHECK: Other: 0
-------------- next part --------------
A non-text attachment was scrubbed...
Name: D2329.1.patch
Type: text/x-patch
Size: 2773 bytes
Desc: not available
URL: <http://lists.llvm.org/pipermail/llvm-commits/attachments/20131204/1ad04753/attachment.bin>


More information about the llvm-commits mailing list
Index: include/llvm/MC/MCELFSymbolFlags.h
===================================================================
--- include/llvm/MC/MCELFSymbolFlags.h
+++ include/llvm/MC/MCELFSymbolFlags.h
@@ -21,10 +21,11 @@
 
 namespace llvm {
   enum {
-    ELF_STT_Shift   = 0, // Shift value for STT_* flags.
-    ELF_STB_Shift   = 4, // Shift value for STB_* flags.
-    ELF_STV_Shift   = 8, // Shift value for STV_* flags.
-    ELF_Other_Shift = 10 // Shift value for other flags.
+    ELF_STT_Shift   = 0,  // Shift value for STT_* flags.
+    ELF_STB_Shift   = 4,  // Shift value for STB_* flags.
+    ELF_STV_Shift   = 8,  // Shift value for STV_* flags.
+    ELF_STO_Shift   = 10, // Shift value for STO_* flags.
+    ELF_Other_Shift = 16  // Shift value for other flags.
   };
 
   enum ELFSymbolFlags {